"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var DefaultStyler_1 = require("./DefaultStyler");
var Util_1 = require("./Util");
var Types_1 = require("./Types");
var styler = DefaultStyler_1.default, globalPrefix = "";
/**
 * Enabled colored output
 *
 * @type {boolean}
 */
var stylerEnabled = false; // true
/**
 * Store explicit category levels
 *
 * @type {TCategoryLevels}
 */
var categoryLevels = {};
function setCategoryLevels(newLevels) {
    Object.assign(categoryLevels, newLevels);
}
exports.setCategoryLevels = setCategoryLevels;
// Load any existing levels
var g = (typeof window !== 'undefined') ? window : (typeof global !== 'undefined') ? global : null;
if (g && g.TypeLoggerCategories) {
    setCategoryLevels(g.TypeLoggerCategories);
}
var logThreshold = g.TypeLoggerDefaultLevel || Types_1.LogLevel.DEBUG;
/**
 * Get the category level
 *
 * @param name
 * @returns {LogLevel|number}
 */
function categoryLevel(name) {
    return categoryLevels[name] || 0;
}
exports.categoryLevel = categoryLevel;
/**
 * Set global threshold
 *
 * @param level
 */
function setLogThreshold(level) {
    logThreshold = level;
}
exports.setLogThreshold = setLogThreshold;
/**
 * Current logger output
 */
var loggerOutput = new Proxy(console, {
    get: function (target, prop) {
        if (prop === 'name' && !target.name)
            return 'console';
        return target[prop];
    }
});
// Does process object exist
var hasProcess = typeof process === 'undefined';
// If in development env then add a few helpers
if (hasProcess && Util_1.getProp(process, 'env.NODE_ENV') === 'development') {
    Object.assign(global, {
        debugCat: function (cat) {
            try {
                process.env.LOG_DEBUG = (process.env.LOG_DEBUG || '') + ',' + cat;
            }
            catch (err) {
                Util_1.warnLog("Failed to set cat debugging: " + cat, err);
            }
        }
    });
}
/**
 * Is debugging enabled for a category
 * based on comma delimited string
 *
 * @param envVal
 * @param name
 * @param delimiter
 * @returns {boolean}
 */
function stringIncludes(envVal, name, delimiter) {
    if (delimiter === void 0) { delimiter = ','; }
    return (!envVal) ? false :
        envVal
            .split(delimiter)
            .map(function (val) { return val.toLowerCase(); })
            .includes(name.toLowerCase());
}
// /**
//  * Check if a category has debugging
//  *
//  * @param name
//  * @returns {boolean}
//  */
// function checkDebug(name:string):boolean {
// 	if (!hasProcess)
// 		return false
//
// 	const envDEBUG = getProp(process,'env.DEBUG'),
// 		envLOG_DEBUG = getProp(process,'env.LOG_DEBUG') || ''
//
//
// 	return stringIncludes(envDEBUG,name) || stringIncludes(envLOG_DEBUG,name)
// }
var overrideLevels = {};
/**
 * Set an override threshold for a logger
 *
 * @param logger
 * @param overrideLevel
 */
function setOverrideLevel(logger, overrideLevel) {
    overrideLevels[logger.name] = overrideLevel;
}
exports.setOverrideLevel = setOverrideLevel;
/**
 * Generic log action
 *
 * @param logger
 * @param name
 * @param level
 * @param args
 */
function log(logger, name, level) {
    var args = [];
    for (var _i = 3; _i < arguments.length; _i++) {
        args[_i - 3] = arguments[_i];
    }
    var overrideLevel = overrideLevels[logger.name];
    if (typeof overrideLevel !== 'number')
        overrideLevel = -1;
    var 
    // debugEnabled = checkDebug(name),
    msgLevel = Util_1.parseLogLevel(level), catLevel = categoryLevel(name);
    if (overrideLevel > msgLevel || (overrideLevel === -1 && ((catLevel > 0 && msgLevel < catLevel) || (catLevel < 1 && msgLevel < logThreshold))))
        return;
    var logOut = loggerOutput, logFns = [
        logOut[level] ? function () {
            var msgArgs = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                msgArgs[_i] = arguments[_i];
            }
            logOut[level].apply(logOut, msgArgs);
        } : null,
        logOut.log ? logOut.log.bind(logOut) : null,
        logOut
    ];
    var logFn = null;
    for (var _a = 0, logFns_1 = logFns; _a < logFns_1.length; _a++) {
        logFn = logFns_1[_a];
        if (logFn && typeof logFn === 'function')
            break;
    }
    if (!logFn) {
        throw new Error('Logger output can not be null');
    }
    var textMsg = Util_1.formatValue(args.shift());
    if (stylerEnabled && styler) {
        styler.apply(void 0, [logFn, "" + (globalPrefix || "") + name, level].concat(args));
    }
    else {
        logFn.apply(void 0, [(globalPrefix || "") + "[" + name + "] [" + level.toUpperCase() + "] " + textMsg].concat(args));
    }
}
/**
 * Default log factory, uses console
 */
exports.DefaultLoggerFactory = {
    /**
     * Creates a simple logger, parsing
     * provided category as a simple filename
     * and using the current output for output
     *
     * @param name
     * @returns {ILogger}
     */
    create: function (name) {
        name = name.split('/').pop().split('.').shift();
        var logger = { name: name };
        // Create levels
        Types_1.LogLevelNames.reduce(function (logger, level) {
            logger[level] = function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                log.apply(void 0, [logger, name, level].concat(args));
            };
            return logger;
        }, logger);
        // ADD THE OVERRIDE FUNCTION
        logger.setOverrideLevel = function (level) {
            setOverrideLevel(logger, level);
        };
        return logger;
    }
};
/**
 * Internal core logger factory
 */
var loggerFactory = exports.DefaultLoggerFactory;
/**
 * Change the internal default logger
 *
 * @export
 * @param newLoggerFactory new logger factory
 */
function setLoggerFactory(newLoggerFactory) {
    loggerFactory = newLoggerFactory;
}
exports.setLoggerFactory = setLoggerFactory;
/**
 * Set the logger output
 *
 * @param newLoggerOutput
 */
function setLoggerOutput(newLoggerOutput) {
    loggerOutput = newLoggerOutput;
}
exports.setLoggerOutput = setLoggerOutput;
/**
 * Set the styler
 *
 * @param newStyler
 */
function setStyler(newStyler) {
    styler = newStyler;
}
exports.setStyler = setStyler;
/**
 * Get the current styler
 *
 * @returns {ILogStyler}
 */
function getStyler() {
    return styler;
}
exports.getStyler = getStyler;
/**
 * Global prefix for all loggers
 *
 * @param prefix
 */
function setPrefixGlobal(prefix) {
    globalPrefix = prefix;
}
exports.setPrefixGlobal = setPrefixGlobal;
/**
 * Wrap a logger and prepend all log calls
 *
 * @param logger
 * @param prefix
 */
function setPrefix(logger, prefix) {
    var newLogger = Object.assign({}, logger);
    Types_1.LogLevelNames.forEach(function (level) {
        /**
         * (description)
         *
         * @param args (description)
         */
        newLogger[level] = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            return logger[level].apply(logger, [prefix].concat(args));
        };
    });
    return newLogger;
}
exports.setPrefix = setPrefix;
/**
 * Set custom styler enabled
 *
 * @param enabled
 */
function setStylerEnabled(enabled) {
    stylerEnabled = enabled;
}
exports.setStylerEnabled = setStylerEnabled;
/**
 * Create a new logger
 *
 * @param name
 * @returns {ILogger}
 */
function create(name) {
    return loggerFactory.create(name);
}
exports.create = create;
//# sourceMappingURL=Logger.js.map